@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2015 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -11,8 +10,6 @@ logger = logging.getLogger('zim.notebook')
 
 
 from zim.parsing import link_type
-from zim.formats.wiki import WIKI_FORMAT_VERSION # FIXME hard coded preference for wiki format
-
 
 import zim.formats
 import zim.fs
@@ -26,7 +23,7 @@ import zim.datetimetz as datetime
 _pagename_reduce_colon_re = re.compile('::+')
 _pagename_invalid_char_re = re.compile(
 	'(' +
-		'^[_\W]|(?<=:)[_\W]' +
+		'^[_\W]+|(?<=:)[_\W]+' +
 	'|' +
 		'[' + re.escape(''.join(
 			("?", "#", "/", "\\", "*", '"', "<", ">", "|", "%", "\t", "\n", "\r")
@@ -34,7 +31,7 @@ _pagename_invalid_char_re = re.compile(
 	')',
 re.UNICODE)
 	# This pattern matches a non-alphanumber at start or after the ':'
-	# seperator. It also matches any invalid character.
+	# separator. It also matches any invalid character.
 	# The UNICODE flag is used to make the alphanumber check international.
 
 
@@ -60,7 +57,7 @@ def shortest_unique_names(paths):
 			conflicts.insert(0, path) # shuffle path of interest to front
 			reverse_paths = [reversed(p.name.split(':')) for p in conflicts]
 			names = []
-			for parts in itertools.izip_longest(*reverse_paths):
+			for parts in itertools.zip_longest(*reverse_paths):
 				if parts[0] is None:
 					break
 				elif parts[0] not in parts[1:]:
@@ -81,7 +78,7 @@ class Path(object):
 	of the page and is used instead of the actual page object by methods
 	that only need to know the name of the page. Path objects have no
 	internal state and are essentially normalized page names. It also
-	has a number of methods to compare page names and determing what
+	has a number of methods to compare page names and determining what
 	the parent pages are etc.
 
 	@ivar name: the full name of the path
@@ -134,7 +131,7 @@ class Path(object):
 		@param name: a string
 		@raises AssertionError: if the name is not valid
 		'''
-		assert isinstance(name, basestring)
+		assert isinstance(name, str)
 		if not name.strip(':') \
 		or _pagename_reduce_colon_re.search(name) \
 		or _pagename_invalid_char_re.search(name):
@@ -179,7 +176,7 @@ class Path(object):
 			self.name = name.strip(':')
 
 		try:
-			self.name = unicode(self.name)
+			self.name = str(self.name)
 		except UnicodeDecodeError:
 			raise ValueError('BUG: invalid input, page names should be in ascii, or given as unicode')
 
@@ -293,13 +290,19 @@ class Path(object):
 		return Path(self.name + ':' + basename)
 
 	def ischild(self, parent):
-		'''Check of this path is a child of a given path
-
+		'''Check whether this path is a child of a given path
 		@param parent: a L{Path} object
 		@returns: True when this path is a (grand-)child of C{parent}
 		'''
 		return parent.isroot or self.name.startswith(parent.name + ':')
 
+	def match_namespace(self, namespace):
+		'''Check whether this path is in a specific section of the notebook
+		@param namespace: a L{Path} object
+		@returns: True when this path is equal to C{namespace} or is a (grand-)child of C{namespace}
+		'''
+		return namespace.isroot or self.name == namespace.name or self.name.startswith(namespace.name + ':')
+
 	def commonparent(self, other):
 		'''Find a common parent for two Paths
 
@@ -502,22 +505,34 @@ class Page(Path, SignalEmitter):
 			self._last_etag = None
 			self._meta = None
 
+	def check_source_changed(self):
+		return self._check_source_etag()
+
 	def _check_source_etag(self):
 		if (
 			self._last_etag
 			and not self.source_file.verify_etag(self._last_etag)
 		) or (
 			not self._last_etag
-			and self._parsetree
+			and (self._parsetree or self._ui_object)
 			and self.source_file.exists()
 		):
 			logger.info('Page changed on disk: %s', self.name)
 			self._last_etag = None
 			self._meta = None
 			self._parsetree = None
+			if self._ui_object is not None:
+				obj = self._ui_object
+				self._ui_object = None
+				parsetree = self.get_parsetree()
+				self._ui_object = obj
+				self._parsetree = None
+				self._ui_object.set_parsetree(parsetree)
+
 			self.emit('page-changed', True)
+			return True
 		else:
-			pass # no check
+			return False
 
 	def exists(self):
 		'''C{True} when the page has either content or children'''
@@ -559,7 +574,7 @@ class Page(Path, SignalEmitter):
 				return None
 			else:
 				parser = self.format.Parser()
-				self._parsetree = parser.parse(text)
+				self._parsetree = parser.parse(text, file_input=True)
 				self._meta = self._parsetree.meta
 				assert self._meta is not None
 				return self._parsetree
@@ -629,7 +644,7 @@ class Page(Path, SignalEmitter):
 
 		@returns: text as a list of lines or an empty list
 		'''
-		if isinstance(format, basestring):
+		if isinstance(format, str):
 			format = zim.formats.get_format(format)
 
 		if not linker is None:
@@ -653,7 +668,7 @@ class Page(Path, SignalEmitter):
 		@param append: if C{True} the text is appended instead of
 		replacing current content.
 		'''
-		if isinstance(format, basestring):
+		if isinstance(format, str):
 			format = zim.formats.get_format(format)
 
 		if append:
@@ -711,7 +726,7 @@ class Page(Path, SignalEmitter):
 	def get_title(self):
 		tree = self.get_parsetree()
 		if tree:
-			return tree.get_heading() or self.basename
+			return tree.get_heading_text() or self.basename
 		else:
 			return self.basename
 
@@ -723,6 +738,6 @@ class Page(Path, SignalEmitter):
 		'''
 		tree = self.get_parsetree()
 		if tree:
-			return tree.get_heading() == self.basename
+			return tree.get_heading_text() == self.basename
 		else:
 			return False
